@@ -1,6 +1,6 @@
 <?php
 /**
- * Copyright 2010-2015 Horde LLC (http://www.horde.org/)
+ * Copyright 2010-2016 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
@@ -10,6 +10,8 @@
  * @author   Gunnar Wrobel <wrobel@pardus.de>
  * @license  http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @link     http://pear.horde.org/index.php?package=Kolab_Storage
+ * @deprecated This class is not maintained and will be removed in future
+ *             releases.
  */
 
 /**
@@ -437,7 +439,8 @@ extends Horde_Kolab_Storage_Driver_Base
      *
      * @param string $folder Check the status of this folder.
      *
-     * @return array  An array that contains 'uidvalidity' and 'uidnext'.
+     * @return array  An array that contains 'uidvalidity', 'uidnext', and
+     *                'token'.
      */
     public function status($folder)
     {
@@ -460,7 +463,8 @@ extends Horde_Kolab_Storage_Driver_Base
         }
         return array(
             'uidvalidity' => $result['UIDVALIDITY'],
-            'uidnext' => $result['UIDNEXT']
+            'uidnext' => $result['UIDNEXT'],
+            'token' => false
         );
     }
 
