@@ -15,7 +15,7 @@
 /**
  * Test the cached data handler.
  *
- * Copyright 2011-2015 Horde LLC (http://www.horde.org/)
+ * Copyright 2011-2016 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
@@ -79,7 +79,7 @@ extends Horde_Kolab_Storage_TestCase
     public function testDataQueriable()
     {
         $data = $this->_getDataCache();
-        $this->assertTrue($data instanceOf Horde_Kolab_Storage_Queriable);
+        $this->assertTrue($data instanceof Horde_Kolab_Storage_Queriable);
     }
 
     public function testGetObjects()
